@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2014 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2015 Eric Lafortune @ GuardSquare
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -203,7 +203,12 @@ implements   ClassVisitor,
             // Update the String entry if required.
             if (!newInternalClassName.equals(internalClassName))
             {
-                String newExternalClassName = ClassUtil.externalClassName(newInternalClassName);
+                // Only convert to an external class name if the original was
+                // an external class name too.
+                String newExternalClassName =
+                    externalClassName.indexOf(JavaConstants.PACKAGE_SEPARATOR) >= 0 ?
+                        ClassUtil.externalClassName(newInternalClassName) :
+                        newInternalClassName;
 
                 // Refer to a new Utf8 entry.
                 stringConstant.u2stringIndex =
