@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2014 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2015 Eric Lafortune @ GuardSquare
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -572,11 +572,10 @@ implements   ClassVisitor,
         // Shift the used constant pool entries together.
         for (int index = 1; index < length; index++)
         {
-            constantIndexMap[index] = counter;
-
             Constant constant = constantPool[index];
 
-            // Don't update the flag if this is the second half of a long entry.
+            // Is the constant being used? Don't update the flag if this is the
+            // second half of a long entry.
             if (constant != null)
             {
                 isUsed = isUsed(constant);
@@ -584,8 +583,17 @@ implements   ClassVisitor,
 
             if (isUsed)
             {
+                // Remember the new index.
+                constantIndexMap[index] = counter;
+
+                // Shift the constant pool entry.
                 constantPool[counter++] = constant;
             }
+            else
+            {
+                // Remember an invalid index.
+                constantIndexMap[index] = -1;
+            }
         }
 
         // Clear the remaining constant pool elements.
