@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2014 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2015 Eric Lafortune @ GuardSquare
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -71,6 +71,7 @@ implements   AttributeVisitor,
     private boolean[]                generalizedContexts  = new boolean[ClassConstants.TYPICAL_CODE_LENGTH];
     private int[]                    evaluationCounts     = new int[ClassConstants.TYPICAL_CODE_LENGTH];
     private boolean                  evaluateExceptions;
+    private int                      codeLength;
 
     private final BasicBranchUnit    branchUnit;
     private final BranchTargetFinder branchTargetFinder;
@@ -299,7 +300,7 @@ implements   AttributeVisitor,
                 if (isTraced(offset))
                 {
                     int initializationOffset = branchTargetFinder.initializationOffset(offset);
-                    if (initializationOffset != NONE)
+                    if (initializationOffset >= 0)
                     {
                         System.out.println("     is to be initialized at ["+initializationOffset+"]");
                     }
@@ -1111,30 +1112,30 @@ implements   AttributeVisitor,
      */
     private void initializeArrays(CodeAttribute codeAttribute)
     {
-        int codeLength = codeAttribute.u4codeLength;
+        int newCodeLength = codeAttribute.u4codeLength;
 
         // Create new arrays for storing information at each instruction offset.
-        if (variablesAfter.length < codeLength)
+        if (branchOriginValues.length < newCodeLength)
         {
             // Create new arrays.
-            branchOriginValues  = new InstructionOffsetValue[codeLength];
-            branchTargetValues  = new InstructionOffsetValue[codeLength];
-            variablesBefore     = new TracedVariables[codeLength];
-            stacksBefore        = new TracedStack[codeLength];
-            variablesAfter      = new TracedVariables[codeLength];
-            stacksAfter         = new TracedStack[codeLength];
-            generalizedContexts = new boolean[codeLength];
-            evaluationCounts    = new int[codeLength];
+            branchOriginValues  = new InstructionOffsetValue[newCodeLength];
+            branchTargetValues  = new InstructionOffsetValue[newCodeLength];
+            variablesBefore     = new TracedVariables[newCodeLength];
+            stacksBefore        = new TracedStack[newCodeLength];
+            variablesAfter      = new TracedVariables[newCodeLength];
+            stacksAfter         = new TracedStack[newCodeLength];
+            generalizedContexts = new boolean[newCodeLength];
+            evaluationCounts    = new int[newCodeLength];
         }
         else
         {
-            // Reset the arrays.
-            Arrays.fill(branchOriginValues,  null);
-            Arrays.fill(branchTargetValues,  null);
-            Arrays.fill(generalizedContexts, false);
-            Arrays.fill(evaluationCounts,    0);
+            // Reset the old arrays.
+            Arrays.fill(branchOriginValues,  0, codeLength, null);
+            Arrays.fill(branchTargetValues,  0, codeLength, null);
+            Arrays.fill(generalizedContexts, 0, codeLength, false);
+            Arrays.fill(evaluationCounts,    0, codeLength, 0);
 
-            for (int index = 0; index < codeLength; index++)
+            for (int index = 0; index < newCodeLength; index++)
             {
                 if (variablesBefore[index] != null)
                 {
@@ -1156,7 +1157,32 @@ implements   AttributeVisitor,
                     stacksAfter[index].reset(codeAttribute.u2maxStack);
                 }
             }
+
+            for (int index = newCodeLength; index < codeLength; index++)
+            {
+                if (variablesBefore[index] != null)
+                {
+                    variablesBefore[index].reset(0);
+                }
+
+                if (stacksBefore[index] != null)
+                {
+                    stacksBefore[index].reset(0);
+                }
+
+                if (variablesAfter[index] != null)
+                {
+                    variablesAfter[index].reset(0);
+                }
+
+                if (stacksAfter[index] != null)
+                {
+                    stacksAfter[index].reset(0);
+                }
+            }
         }
+
+        codeLength = newCodeLength;
     }
 
 
