@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2014 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2015 Eric Lafortune @ GuardSquare
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -635,12 +635,10 @@ implements   ClassVisitor,
 
             if (referencedClasses != null)
             {
-                int returnedClassIndex =
-                    new DescriptorClassEnumeration(descriptor).classCount() - 1;
+                Clazz referencedClass =
+                    referencedClasses[referencedClasses.length - 1];
 
-                Clazz returnedClass = referencedClasses[returnedClassIndex];
-
-                return isSimpleEnum(returnedClass);
+                return isSimpleEnum(referencedClass);
             }
         }
 
