@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2014 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2015 Eric Lafortune @ GuardSquare
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -71,18 +71,32 @@ implements   ClassVisitor,
         // Change the references of the attributes.
         programClass.attributesAccept(this);
 
-        // Remove interface classes that have ended up pointing to the class itself.
-        int newInterfacesCount = 0;
+        // Remove duplicate interfaces and interface classes that have ended
+        // up pointing to the class itself.
+        boolean[] delete = null;
         for (int index = 0; index < programClass.u2interfacesCount; index++)
         {
             Clazz interfaceClass = programClass.getInterface(index);
-            if (!programClass.equals(interfaceClass))
+            if (interfaceClass != null &&
+                (programClass.equals(interfaceClass) ||
+                 containsInterfaceClass(programClass,
+                                        index,
+                                        interfaceClass)))
             {
-                programClass.u2interfaces[newInterfacesCount++] =
-                    programClass.u2interfaces[index];
+                // Lazily create the array.
+                if (delete == null)
+                {
+                    delete = new boolean[programClass.u2interfacesCount];
+                }
+
+                delete[index] = true;
             }
         }
-        programClass.u2interfacesCount = newInterfacesCount;
+
+        if (delete != null)
+        {
+            new InterfaceDeleter(delete).visitProgramClass(programClass);
+        }
 
         // Is the class being retargeted?
         Clazz targetClass = ClassMerger.getTargetClass(programClass);
@@ -192,6 +206,13 @@ implements   ClassVisitor,
     }
 
 
+    public void visitInvokeDynamicConstant(Clazz clazz, InvokeDynamicConstant invokeDynamicConstant)
+    {
+        // Change the referenced classes.
+        updateReferencedClasses(invokeDynamicConstant.referencedClasses);
+    }
+
+
     public void visitAnyRefConstant(Clazz clazz, RefConstant refConstant)
     {
         Clazz referencedClass    = refConstant.referencedClass;
@@ -385,7 +406,27 @@ implements   ClassVisitor,
 
     // Small utility methods.
 
-    /**
+     /**
+     * Returns whether the given class contains the given interface
+     * class in its first given number of interfaces.
+     */
+    private boolean containsInterfaceClass(Clazz clazz,
+                                           int   interfaceCount,
+                                           Clazz interfaceClass)
+    {
+        for (int index = 0; index < interfaceCount; index++)
+        {
+            if (interfaceClass.equals(clazz.getInterface(index)))
+            {
+                return true;
+            }
+        }
+
+        return false;
+    }
+
+
+   /**
      * Updates the retargeted classes in the given array of classes.
      */
     private void updateReferencedClasses(Clazz[] referencedClasses)
